@@ -9,6 +9,7 @@ import re
 import logging
 
 import requests
+import requests.utils
 import git
 import toml
 
@@ -27,6 +28,11 @@ sites = {
     GITHUB: 'github',
 }
 
+SITE_ABBREVS = {
+    'bitbucket': 'bitbucket',
+    'bb':        'bitbucket',
+    'github':    'github',
+    'gh':        'github'}
 
 CONFIG_USAGE = '''\
 Provide a config file with --config argument or its default location: '~/.config/ripio'.
@@ -93,9 +99,12 @@ class BadWorkspaceName(error):
 
 class UnsupportedSite(error): pass
 
-class DestinationDirectoryAlreadyExists(error):
+class DirectoryAlreadyExists(error):
     reason = 'destination directory already EXISTS'
 
+class DirectoryIsNotRepository(error):
+    reason = 'destination directory exists but is NOT a git repository'
+
 class WrongCompletion(error): pass
 
 class AccessDenied(error):
@@ -104,6 +113,22 @@ class AccessDenied(error):
 
 class AlreadyExists(error): pass
 
+class MissingRemoteOrigin(error):
+    reason = "missing remote 'origin' for repository"
+
+class RepositoryAlreadyCloned(error):
+    reason = "repository already cloned"
+
+class UnrelatedRepository(error):
+    reason = "unrelated destination directory"
+
+    def __init__(self, destdir, repo_ref):
+        assert isinstance(repo_ref, RepoRef)
+        self.value = str(destdir), str(repo_ref)
+
+class RateLimitExceeded(error):
+    reason = "Site rate limit exceeded. Too many requests!"
+
 
 class WorkspaceName:
     def __init__(self, full_workspace, site=None):
@@ -119,11 +144,9 @@ class WorkspaceName:
             self.site = site
             self.workspace = full_workspace
 
-        abbrevs = {'bb': 'bitbucket', 'gh': 'github'}
-        if self.site in abbrevs:
-            self.site = abbrevs[self.site]
-
-        if self.site not in ['bitbucket', 'github']:
+        try:
+            self.site = SITE_ABBREVS[self.site]
+        except KeyError:
             raise UnsupportedSite(self.site)
 
     def __repr__(self):
@@ -135,6 +158,9 @@ class WorkspaceName:
 
 class RepoRef:
     def __init__(self, site_full_name, site=None):
+        if site_full_name.startswith(('https://', 'git@')):
+            site_full_name = RepoRef.parse_origin(site_full_name)
+
         if site_full_name.count('/') != 1:
             raise BadRepositoryName(site_full_name)
 
@@ -149,14 +175,21 @@ class RepoRef:
         self.global_name = '{}:{}'.format(self.site, self.full_name)
 
     @classmethod
-    def from_origin(self, path):
-        if path.startswith('git@'):
-            fields = re.findall(r'\Agit@([^:]+):(.+).git\Z', path)[0]
-            return RepoRef('{}:{}'.format(sites[fields[0]], fields[1]))
+    def parse_origin(cls, url):
+        if url.startswith('git@'):
+            fields = re.findall(r'\Agit@([^:]+):(.+).git\Z', url)[0]
+            return '{}:{}'.format(sites[fields[0]], fields[1])
 
-        elif path.startswith('https://'):
-            fields = re.findall(r'\Ahttps?://([^/]+)/(.+).git\Z', path)[0]
-            return RepoRef('{}:{}'.format(sites[fields[0]], fields[1]))
+        elif url.startswith('https://'):
+            print(url)
+            fields = re.findall(r'\Ahttps?://([^/]+)/(.+)(?:\.git)?\Z', url)[0]
+            return '{}:{}'.format(sites[fields[0]], fields[1])
+
+        raise BadRepositoryName(url)
+
+    @classmethod
+    def from_origin(cls, url):
+        return RepoRef(cls.parse_origin(url))
 
     @classmethod
     def from_parts(cls, workspace, name, site=None):
@@ -203,13 +236,22 @@ class Completion:
         except BadRepositoryName:
             pass
 
-        for ws in self.workspaces:
+        workspaces = self.workspaces[:]
+
+        # for site:name names
+        if name.count(':') == 1:
+            print(name)
+            site, name = name.split(':')
+            workspaces = [ws for ws in workspaces if ws.site == SITE_ABBREVS[site]]
+            print(workspaces)
+
+        for ws in workspaces:
             try:
                 repo = ws.make_repo(name)
                 if repo.exists():
-                    self.found.append(repo.name.global_name)
+                    self.found.append(repo.ref.global_name)
             except AccessDenied:
-                self.denied.append(repo.name.global_name)
+                self.denied.append(repo.ref.global_name)
 
     @classmethod
     def list_workspaces(cls, config):
@@ -223,8 +265,8 @@ class Completion:
             try:
                 name = credentials.username
                 retval.append(ws_class[site](name, credentials))
-            except AttributeError as e:
-                logging.debug(e)
+            except AttributeError:
+                logging.debug("No credentials found for '{}'".format(site))
 
             try:
                 for name in config.get_workspaces(site):
@@ -320,12 +362,18 @@ class Auth:
         self.credentials = credentials
 
     def auth(self, url):
+        assert url.startswith('https')
+
         if self.credentials is None:
             return url
 
         parts = urlparse(url)
         user_pass = '{}:{}@'.format(self.credentials.username, self.credentials.password)
-        return urlunparse(parts._replace(netloc=user_pass + parts.netloc))
+        netloc = parts.netloc
+        if '@' in netloc:
+            netloc = netloc.split('@')[1]
+
+        return urlunparse(parts._replace(netloc=user_pass + netloc))
 
 
 def safe_url(url):
@@ -343,7 +391,7 @@ def safe_url(url):
 
 class Repo(Auth):
     def check(self):
-        self.data
+        self.webpage
         return True
 
     def exists(self):
@@ -353,9 +401,7 @@ class Repo(Auth):
         except RepositoryNotFound:
             return False
 
-    @property
-    @lru_cache()
-    def data(self):
+    def _load_full_data(self):
         logging.debug(self.url)
 
         # FIXME: catch connection exceptions
@@ -363,20 +409,30 @@ class Repo(Auth):
         self.reply_check(result)
         retval = result.json()
         retval['access'] = self._get_access(retval)
+        retval['size'] = self._get_size(retval)
         return retval
 
+    @lru_cache()
     def __getattr__(self, attr):
-        assert attr in 'scm slug full_name size access'.split(), \
-            "Missing attribute '{}'".format(attr)
-
         try:
-            return self.basic_data[attr]
+            return self._data[attr]
         except KeyError:
-            return self.data[attr]
+            self._data.update(self._load_full_data())
+            try:
+                return self._data[attr]
+            except KeyError:
+                raise AttributeError(attr)
+
 
     @classmethod
     def from_dir(cls, dirname, credentials=None):
-        origin = git.Repo(dirname).remote().url
+        try:
+            origin = git.Repo(dirname).remote().url
+        except git.exc.InvalidGitRepositoryError:
+            raise DirectoryIsNotRepository(dirname)
+        except ValueError:
+            raise MissingRemoteOrigin(dirname)
+
         logging.debug(origin)
         repo_ref = RepoRef.from_origin(origin)
         return cls.make(repo_ref, credentials)
@@ -396,20 +452,46 @@ class Repo(Auth):
         return repo_class(repo_ref, credentials.get(repo_ref.site))
 
     def __repr__(self):
-        return "<{} '{}'>".format(self.__class__.__name__, self.name.global_name)
+        return "<{} '{}'>".format(self.__class__.__name__, self.ref.global_name)
+
+    def __str__(self):
+        return str(self.ref)
 
     def info(self):
         data = dict(
+            webpage=self.webpage,
+            origin_https=self.clone_links['https'],
+            origin_ssh=self.clone_links['ssh'],
             access=self.access,
-            size=utils.to_kB(self.size)
+            size='{:,} KB'.format(self.size)
         )
 
         retval = ""
         for key, val in data.items():
-            retval += f"- {key+':':8} {val:>10}\n"
+            retval += f"- {key.replace('_', ' ')+':':14} {val}\n"
 
         return retval
 
+    def clone(self, destdir, proto='ssh'):
+        def dash(*data):
+            print('-', end='', flush=True)
+
+        url = self.clone_links[proto]
+        if proto == 'https':
+            url = self.auth(url)
+
+        logging.debug(url)
+
+        try:
+            git.Repo.clone_from(url, destdir, progress=dash)
+        except git.exc.GitCommandError as e:
+            logging.error("cloning failed.")
+            if proto == 'https':
+                raise e
+
+            print("- retrying with 'https'...")
+            self.clone(destdir, proto='https')
+
 
 def api_check(reply, expected, raises):
     expected = expected or [200]
@@ -456,6 +538,9 @@ class Bitbucket:
 class Github:
     @classmethod
     def api_check(cls, reply, expected=None, raises=None):
+        raises = raises or {}
+        raises[403] = RateLimitExceeded(GITHUB)
+
         msg = api_check(reply, expected, raises)
         if msg is None:
             return
@@ -474,38 +559,44 @@ class BitbucketRepo(Repo):
     BASE_URL = 'https://api.bitbucket.org/2.0/repositories/{owner}/{repo}'
 
     def __init__(self, name, credentials=None):
-        self.name = RepoRef.cast(name, site='bitbucket')
+        self.ref = RepoRef.cast(name, site='bitbucket')
 
         super().__init__(credentials)
-        self.basic_data = dict(
-            full_name=self.name.full_name,
-            slug=self.name.slug)
+        self._data = dict(
+            full_name=self.ref.full_name,
+            slug=self.ref.slug)
 
         self.url = self.auth(self.BASE_URL.format(
-            owner=self.name.owner.workspace, repo=self.name.slug))
+            owner=self.ref.owner.workspace, repo=self.ref.slug))
 
     def reply_check(self, reply, expected=None, raises=None):
         raises = raises or {}
         raises.update({
-            403: AccessDenied(self.name),
-            404: RepositoryNotFound(self.name)
+            403: AccessDenied(self.ref),
+            404: RepositoryNotFound(self.ref)
         })
         Bitbucket.api_check(reply, expected, raises=raises)
 
     @classmethod
     def from_data(cls, data, credentials=None):
         instance = cls(data['full_name'], credentials)
-        instance.basic_data = dict(
+        instance._data.update(dict(
             scm=data['scm'],
             slug=data['slug'],
             full_name=data['full_name'],
-            size=data['size'],
-            access='private' if data['is_private'] else 'public')
+            size=cls._get_size(data),
+            access=cls._get_access(data)
+        ))
         return instance
 
-    def _get_access(self, data):
+    @classmethod
+    def _get_access(cls, data):
         return 'private' if data['is_private'] else 'public'
 
+    @classmethod
+    def _get_size(cls, data):
+        return float(data['size']) / 1000
+
     @property
     @lru_cache()
     def clone_links(self):
@@ -521,14 +612,14 @@ class BitbucketRepo(Repo):
             }
         ]'''
         retval = {}
-        for link in self.data['links']['clone']:
+        for link in self.links['clone']:
             retval[link['name']] = link['href']
         return retval
 
     @property
     @lru_cache()
     def webpage(self):
-        return self.data['links']['html']['href']
+        return self.links['html']['href']
 
     def last_commits(self, max_=3):
         commits_url = self.url + '/commits'
@@ -543,10 +634,10 @@ class BitbucketRepo(Repo):
                 date    = c['date'],
                 message = c['message'])
 
-    def create(self):
-        result = requests.post(self.url, data={'is_private':True})
+    def create(self, private=True):
+        result = requests.post(self.url, data={'is_private':private})
         self.reply_check(result, raises={
-            400: AlreadyExists(self.name)
+            400: AlreadyExists(self.ref)
         })
         return self.slug
 
@@ -565,14 +656,24 @@ class BitbucketRepo(Repo):
         real_name = result.headers['Location'].split('/')[-1]
         return real_name
 
-    def clone(self, destdir, proto='ssh'):
-        def dash(*data):
-            print('-', end='', flush=True)
+    @property
+    @lru_cache()
+    def permissions(self):
+        URL = 'https://api.bitbucket.org/2.0/user/permissions/repositories?q=repository.name="{}"'
+        url = self.auth(URL.format(self.ref.slug))
+        print(url)
+        result = requests.get(url)
+        print(self.ref)
+        self.reply_check(result)
+        result = result.json()
 
-        url = self.clone_links[proto]
-        logging.debug(url)
-        git.Repo.clone_from(url, destdir, progress=dash)
-        print()
+        if len(result['values']) == 0:
+            return 'none'
+
+        print(len(result['values']), result['values'])
+
+        result = [r for r in result['values'] if r['repository']['full_name'] == self.ref.full_name][0]
+        return result['permission']
 
 
 class GithubRepo(Repo):
@@ -582,50 +683,56 @@ class GithubRepo(Repo):
 
     # FIXME: refactor superclass
     def __init__(self, name, credentials=None):
-        self.name = RepoRef.cast(name, site='github')
+        self.ref = RepoRef.cast(name, site='github')
 
         super().__init__(credentials)
-        self.basic_data = dict(
-            full_name=self.name.full_name,
-            slug=self.name.slug)
+        self._data = dict(
+            full_name=self.ref.full_name,
+            slug=self.ref.slug)
 
         self.url = self.auth(self.BASE_URL.format(
-            owner=self.name.owner.workspace, repo=self.name.slug))
+            owner=self.ref.owner.workspace, repo=self.ref.slug))
 
     def reply_check(self, reply, expected=None, raises=None):
         raises = raises or {}
         raises.update({
-            401: AccessDenied(self.name),
-            404: RepositoryNotFound(self.name)
+            401: AccessDenied(self.ref),
+            404: RepositoryNotFound(self.ref)
         })
         Github.api_check(reply, expected, raises=raises)
 
     @classmethod
     def from_data(cls, data, credentials=None):
         instance = cls(data['full_name'], credentials)
-        instance.basic_data = dict(
+        instance._data.update(dict(
             scm='git',
             slug=data['name'],
             full_name=data['full_name'],
-            size=data['size'],
-            access='private' if data['private'] else 'public')
+            size=cls._get_size(data),
+            access=cls._get_access(data)
+        ))
         return instance
 
+    @classmethod
     def _get_access(self, data):
         return 'private' if data['private'] else 'public'
 
+    @classmethod
+    def _get_size(cls, data):
+        return float(data['size'])
+
     @property
     @lru_cache()
     def clone_links(self):
         return dict(
-            ssh=self.data['ssh_url'],
-            https=self.data['clone_url']
+            ssh=self.ssh_url,
+            https=self.clone_url
         )
 
     @property
     @lru_cache()
     def webpage(self):
-        return self.data['html_url']
+        return self.html_url
 
     def last_commits(self, max_=3):
         # https://developer.github.com/v3/repos/commits/#list-commits
@@ -648,21 +755,22 @@ class GithubRepo(Repo):
                 message = c['commit']['message'])
 
     def get_user_url(self):
-        if self.name.owner.workspace.lower() == self.credentials.username.lower():
+        if self.credentials and \
+            self.ref.owner.workspace.lower() == self.credentials.username.lower():
             return self.OWNER_URL
 
-        return self.ORG_URL.format(org=self.name.owner.workspace)
+        return self.ORG_URL.format(org=self.ref.owner.workspace)
 
-    def create(self):
+    def create(self, private=True):
         url = self.auth(self.get_user_url())
         logging.debug(url)
 
         result = requests.post(
             url,
-            data=json.dumps({'name': self.slug, 'private': True}))
+            data=json.dumps({'name': self.slug, 'private': private}))
 
         self.reply_check(result, [201], raises={
-            422: AlreadyExists(self.name)
+            422: AlreadyExists(self.ref)
         })
         real_name = result.json()['name']
         return real_name
@@ -688,15 +796,6 @@ class GithubRepo(Repo):
         real_name = result.json()['name'].split('/')[-1]
         return real_name
 
-    def clone(self, destdir, proto='ssh'):
-        def dash(*data):
-            print('-', end='', flush=True)
-
-        url = self.clone_links[proto]
-        logging.debug(url)
-        git.Repo.clone_from(url, destdir, progress=dash)
-        print()
-
 
 class Workspace(Auth):
     def __str__(self):
@@ -707,6 +806,7 @@ class Workspace(Auth):
 
 
 class BitbucketWorkspace(Workspace):
+    site = 'bitbucket'
     BASE_URL = 'https://api.bitbucket.org/2.0/repositories/{}?sort=slug'
 
     def __init__(self, name, credentials):
@@ -740,6 +840,7 @@ class BitbucketWorkspace(Workspace):
 
 
 class GithubWorkspace(Workspace):
+    site = 'github'
     ORG_URL   = 'https://api.github.com/orgs/{org}/repos'
     USER_URL  = 'https://api.github.com/users/{user}/repos'
     OWNER_URL = 'https://api.github.com/user/repos'
@@ -758,7 +859,8 @@ class GithubWorkspace(Workspace):
             self.url = self.get_user_url()
 
     def get_user_url(self):
-        if self.name.workspace.lower() == self.credentials.username.lower():
+        if self.credentials \
+            and self.name.workspace.lower() == self.credentials.username.lower():
             return self.OWNER_URL + '?type=owner'
 
         return self.USER_URL.format(user=self.name.workspace)
