@@ -29,6 +29,7 @@ def get_repo(config, name=None, guess=True):
         if not guess:
             raise
 
+        # FIXME: refactor this handler
         print(f"- trying to complete '{name}' at known workspaces...")
         completion = ripio.Completion(name, config.config_file)
 
@@ -45,18 +46,26 @@ def get_repo(config, name=None, guess=True):
     return ripio.Repo.make(repo_ref, config.credentials)
 
 
+def cwd_repo(config):
+    root = ripio.utils.find_in_ancestors('.git', str(Path.cwd()))
+    return ripio.Repo.from_dir(root, config.credentials)
+
+
 def cmd_ls_repos(config):
     ws_name = ripio.WorkspaceName(config.owner)
-    if ws_name.site == 'bitbucket':
-        ws = ripio.BitbucketWorkspace(ws_name, config.credentials.get('bitbucket'))
-    elif ws_name.site == 'github':
-        ws = ripio.GithubWorkspace(ws_name, config.credentials.get('github'))
-    else:
+    if ws_name.site not in ['bitbucket', 'github']:
         raise ripio.UnsupportedSite(ws_name.site)
 
+    credentials = config.credentials.get(ws_name.site)
+
+    if ws_name.site == 'bitbucket':
+        ws = ripio.BitbucketWorkspace(ws_name, credentials)
+    else:
+        ws = ripio.GithubWorkspace(ws_name, credentials)
+
+    fmt = "{0:>4}. {1:>12,} KB - {2.scm:<3} - {2.access:<7} - {2.full_name:<20}"
     for i, repo in enumerate(ws.ls_repos()):
-        print("{0:>4}. {1:>10} - {2.scm:<3} - {2.access:<7} - {2.full_name:<20}".format(
-            i+1, utils.to_kB(repo.size), repo))
+        print(fmt.format(i+1, repo.size, repo))
 
 
 def cmd_print_head(config):
@@ -75,64 +84,82 @@ def cmd_repo_rename(config):
     repo = get_repo(config)
     new_name = repo.rename(config.new_name)
     print("- repository '{}' renamed as '{}/{}'".format(
-        repo.name.full_name, repo.name.owner.workspace, new_name))
+        repo.ref.full_name, repo.ref.owner.workspace, new_name))
 
 
 def cmd_repo_create(config):
     repo = get_repo(config, guess=False)
-    name = repo.create()
-    print("- repository '{}' created".format(name))
+    repo.create(config.private)
+    print("- repository '{}' created".format(repo))
+
+    if config.clone:
+        config.proto = 'ssh'  # FIXME: add to config file
+        _clone(config, repo)
 
 
 def cmd_repo_delete(config):
     repo = get_repo(config)
     repo.check()
     utils.confirm_irrecoverable_operation()
-    print("- deleting '{}'".format(repo.full_name))
     repo.delete()
+    print("- repository '{}' deleted".format(repo))
 
 
 def cmd_repo_clone(config):
     repo = get_repo(config)
+    _clone(config, repo)
+
+
+def _clone(config, repo):
     destdir = config.destdir / repo.slug
 
     if not repo.exists():
-        raise ripio.RepositoryNotFound(repo.name)
+        raise ripio.RepositoryNotFound(repo.ref)
 
     if Path(destdir).exists():
-        raise ripio.DestinationDirectoryAlreadyExists(destdir)
+        local_clone = ripio.Repo.from_dir(destdir, config.credentials)
+        if local_clone.ref == repo.ref:
+            raise ripio.RepositoryAlreadyCloned(destdir)
+
+        raise ripio.UnrelatedRepository(destdir, local_clone.ref)
 
     print("- cloning({}) '{}' to '{}'".format(
-        config.proto, repo.full_name, utils.pretty_path(destdir)))
+        config.proto, repo, utils.pretty_path(destdir)))
     repo.clone(destdir, config.proto)
+    print()
 
 
 def cmd_show_config(config):
     config = vars(config)
     del config['func']
 
-    print("Command line config:")
+    print("command line config:")
     for key, value in sorted(config.items()):
         if key in ['config_file', 'parser']:
             continue
 
         print(f"- {key}: '{value}'")
 
-    print("\nConfig file:")
+    print("\nconfig file:")
     print(config['config_file'])
+    print("run 'ripio help' for a config file example.")
 
 
 def cmd_site(config):
-    root = ripio.utils.find_in_ancestors('.git', str(Path.cwd()))
-    repo = ripio.Repo.from_dir(root, config.credentials)
+    repo = cwd_repo(config)
     url = repo.webpage
     print("- openning '{}'".format(url))
     webbrowser.open(url)
 
 
 def cmd_info(config):
-    repo = get_repo(config)
+    if config.repo:
+        repo = get_repo(config)
+    else:
+        repo = cwd_repo(config)
+
     print(repo.info())
+#    print(repo.permissions)
 
 
 def cmd_help(config):
@@ -160,19 +187,25 @@ class BaseConfig(argparse.Namespace):
 
 
 def run():
-    repo_help = 'repo ref (site:owner/slug) or name'
+    repo_help = 'repo ref (site:owner/slug), url or name'
 
     parser = argparse.ArgumentParser(
         formatter_class=argparse.RawTextHelpFormatter,
         description='''\
 Manage hosted git repositories.
-General repository name format is: 'site:owner/name'.
-Examples:
-- 'github:twitter/wordpress' or 'gh:twitter/wordpress'
-- 'bitbucket:paypal/exmaple' or 'bb:paypal/exmaple'
 
-Abbreviated names are allowed when suitable configuration is given.
-''')
+General repository reference format is: 'site:owner/name', like:
+- 'github:twitter/wordpress' or 'gh:twitter/wordpress'
+- 'bitbucket:paypal/example' or 'bb:paypal/example'
+
+Abbreviated references are allowed (when 'workspaces' are configured):
+- 'example'
+- 'github:example' o 'gh:example'
+
+URLs are valid too:
+- 'https://github.com/twitter/wordpress'
+- 'git@github.com:twitter/wordpress.git'
+''' + ripio.CONFIG_USAGE)
 
     parser.add_argument('--config', help='alternate config file',
                         default=Path.home() / '.config/ripio')
@@ -209,6 +242,11 @@ Abbreviated names are allowed when suitable configuration is given.
 
     parser_create = cmds.add_parser('create', help='create new repository')
     parser_create.set_defaults(func=cmd_repo_create)
+    parser_create.add_argument('--public', dest='private', default=True,
+                               action='store_false',
+                               help='set public, default is private')
+    parser_create.add_argument('--clone', action='store_true',
+                               help='clone after create')
     parser_create.add_argument('repo', help=repo_help)
 
     parser_delete = cmds.add_parser('delete', help='delete a repository')
@@ -232,7 +270,7 @@ Abbreviated names are allowed when suitable configuration is given.
 
     parser_info = cmds.add_parser('info', help='show repository info')
     parser_info.set_defaults(func=cmd_info)
-    parser_info.add_argument('repo', help=repo_help)
+    parser_info.add_argument('repo', nargs='?', help=repo_help)
 
     config = parser.parse_args(namespace=BaseConfig())
     config.load_file()
